@@ -22,7 +22,7 @@
  * available for blocks (default: 2 MB) */
 #define MAX_BLOCKMEM	(2 * (1 << 20))	
 
-static const char heme_version[] = "0.4";
+static const char heme_version[] = "0.4.2";
 
 typedef unsigned char byte;
 static WINDOW *w_main, *w_offset, *w_ascii;
@@ -35,6 +35,7 @@ static int cur_line;
 static char *file_name;
 static char *prog_name;
 static const char *config_file;
+static int current_nibble;
 
 static int last_search = -1; /* 0 = byte search, 1 = string search */
 static int last_byte_search = 0;
@@ -72,6 +73,8 @@ static void save_undo_info(off_t offset, byte b);
 static int make_backup_files = -1;
 static int advance_after_edit = -1;
 
+static void advance_right();
+
 /* reads file into buf, max n bytes, returns 0 on EOF, < 0 on error */
 static ssize_t read_file(int fds, byte *buf, size_t n)
 {
@@ -284,9 +287,9 @@ static void open_file(const char *name)
 
 static void unhilite()
 {
-	mvwchgat(w_main, cur_line, 3 * (cur_offset % bpl), 2,
+	mvwchgat(w_main, cur_line, (3 * (cur_offset % bpl)), 3,
 			A_NORMAL, use_colors, NULL);
-	mvwchgat(w_ascii, cur_line, cur_offset % bpl, 1,
+	mvwchgat(w_ascii, cur_line, cur_offset % bpl, 3,
 			A_NORMAL, use_colors, NULL);
 	touchwin(w_main);
 	touchwin(w_ascii);
@@ -296,7 +299,7 @@ static void hilite()
 {
 	if(use_colors)
 	{
-		mvwchgat(w_main, cur_line, 3 * (cur_offset % bpl), 2,
+		mvwchgat(w_main, cur_line, (3 * (cur_offset % bpl)) + current_nibble, 1,
 				A_NORMAL, 2, NULL);
 		mvwchgat(w_ascii, cur_line, cur_offset % bpl, 1,
 				A_NORMAL, 2, NULL);
@@ -418,10 +421,19 @@ static void put_status()
 }
 
 /* called when in hex mode */
-static void set_byte_part(byte b, int shift)
+static void set_byte_part(byte b)
 {
 	byte old_byte;
+	int shift;
 
+	if(current_nibble == 1)
+	{
+		shift = 0;
+	}
+	else
+	{
+		shift = 4;
+	}
 	if(isdigit(b))
 		b -= '0';
 	else
@@ -809,20 +821,23 @@ static void main_loop()
 	int i, ret = 0;
 	off_t tmp_offset;
 
-	bpl = COLS > 80 ? 24 : 16;
+	/* determine number of bytes per line (always an even number) */
+	bpl = (COLS - 14) / 4;
+	if(bpl % 2 != 0)
+		bpl--;
 
 	/* lines 0 and (LINES - 1) are reserved for status info */
 	max_lines = LINES - 2;
 
 	w_offset = newwin(max_lines, 13, 1, 0);
-	w_main = newwin(max_lines, COLS - (bpl + 1) - 10, 1, 13);
+	w_main = newwin(max_lines, COLS - (bpl + 1) - 13, 1, 13);
 	w_ascii = newwin(max_lines, bpl + 1, 1, COLS - bpl - 1);
 
 	scrollok(w_offset, TRUE);
 	scrollok(w_main, TRUE);
 	scrollok(w_ascii, TRUE);
 
-	cur_offset = cur_line = beg_offset = 0;
+	current_nibble = cur_offset = cur_line = beg_offset = 0;
 	for(end_offset = file_size; end_offset % bpl != 0; end_offset--)
 		;
 	end_offset -= bpl * (max_lines - 1);
@@ -851,6 +866,16 @@ static void main_loop()
 		wnoutrefresh(w_offset);
 		wnoutrefresh(w_main);
 		wnoutrefresh(w_ascii);
+		if(!mode_ascii)
+		{
+			wmove(w_main, cur_line, (3 * (cur_offset % bpl)) + current_nibble);
+			wnoutrefresh(w_main);
+		}
+		if(mode_ascii)
+		{
+			wmove(w_ascii, cur_line, cur_offset % bpl);
+			wnoutrefresh(w_ascii);
+		}
 		doupdate();
 
 		ch = getch();
@@ -860,7 +885,7 @@ static void main_loop()
 			continue;
 		}
 
-		if(mode_ascii && isprint(ch))
+		if(mode_ascii && isprint(ch) && (0xFF00 & ch) == 0)
 		{
 			set_byte(ch);
 			ch = KEY_RIGHT; /* nice hack for incrementing current_offset ... */
@@ -869,7 +894,7 @@ static void main_loop()
 		switch(ch)
 		{
 		case KEY_LEFT:
-			if(cur_line == 0 && beg_offset == 0 && cur_offset == 0)
+			if(cur_line == 0 && beg_offset == 0 && cur_offset == 0 && (current_nibble == 0 || mode_ascii))
 				break;
 
 			unhilite();
@@ -885,34 +910,27 @@ static void main_loop()
 			}
 			else
 			{
-				cur_offset--;
-				if((cur_offset + 1) % bpl == 0)
+				if(mode_ascii || current_nibble == 0)
+				{
+					cur_offset--;
+					if(mode_ascii) {
+						current_nibble=0;
+					} else {
+						current_nibble=1;
+					}
+				}
+				else
+				{
+					current_nibble--;
+				}
+				if((cur_offset + 1) % bpl == 0 && (current_nibble == 1 || mode_ascii))
 					cur_line--;
 			}
 			hilite();
 			break;
 
 		case KEY_RIGHT:
-			if(cur_offset == file_size - 1)
-				break;
-
-			unhilite();
-			if(cur_line == max_lines - 1 && (cur_offset + 1) % bpl == 0)
-			{
-				/* move down one line */
-				scroll(w_offset);
-				scroll(w_main);
-				scroll(w_ascii);
-				beg_offset += bpl;
-				put_line(max_lines - 1, cur_offset - bpl);
-				cur_offset++;
-			}
-			else
-				cur_offset++;
-
-			if(cur_offset % bpl == 0 && cur_line < max_lines - 1)
-				cur_line++;
-			hilite();
+			advance_right();
 			break;
 
 		case KEY_UP:
@@ -1278,33 +1296,11 @@ static void main_loop()
 			ch = tolower(ch);
 			if(isxdigit(ch))
 			{
-				if(isxdigit(lastch))
+				set_byte_part(ch);
+				if(advance_after_edit)
 				{
-					set_byte_part(ch, 0);
-					ch = EOF;
-					if(advance_after_edit && cur_offset < file_size - 1)
-					{
-						unhilite();
-						if(cur_line == max_lines - 1 && (cur_offset + 1) % bpl == 0)
-						{
-							/* move down one line */
-							scroll(w_offset);
-							scroll(w_main);
-							scroll(w_ascii);
-							beg_offset += bpl;
-							put_line(max_lines - 1, cur_offset - bpl);
-							cur_offset++;
-						}
-						else
-							cur_offset++;
-
-						if(cur_offset % bpl == 0 && cur_line < max_lines - 1)
-							cur_line++;
-						hilite();
-					}
+					advance_right();
 				}
-				else
-					set_byte_part(ch, 4);
 			}
 			break;
 		}
@@ -1323,6 +1319,36 @@ static void main_loop()
 	putchar('\n');
 }
 
+static void advance_right()
+{
+	if(cur_offset == file_size - 1 && (current_nibble == 1 || mode_ascii))
+		return;
+
+	unhilite();
+	if(mode_ascii || current_nibble == 1)
+	{
+		cur_offset++;
+		current_nibble=0;
+	}
+	else
+	{
+		current_nibble++;
+	}
+	if(cur_line == max_lines - 1 && (cur_offset) % bpl == 0 && current_nibble == 0)
+	{
+		/* move down one line */
+		scroll(w_offset);
+		scroll(w_main);
+		scroll(w_ascii);
+		beg_offset += bpl;
+		put_line(max_lines - 1, cur_offset);
+	}
+
+	if(cur_offset % bpl == 0 && current_nibble == 0 && cur_line < max_lines - 1)
+		cur_line++;
+	hilite();
+}
+
 static void init_blocksize()
 {
 
@@ -1421,7 +1447,7 @@ static void setup_curses()
  * defaults will be used
  */
 static pconfig_option_t opts[] = {
-	{ PCONFIG_INT, "advance_after_edit", "0" },
+	{ PCONFIG_INT, "advance_after_edit", "1" },
 	{ PCONFIG_INT, "make_backup_files", "1" },
 	{ PCONFIG_INT, "use_colors", "1" },
 	{ PCONFIG_STRING, "bg_color", "blue" },
@@ -1469,7 +1495,6 @@ static void read_config()
 static void print_usage()
 {
 	printf("Usage: %s [options] FILE\nAvailable options:\n", prog_name);
-	puts("  -a, --advance-after-edit \tadvance after edit");
 	puts("  -n, --no-backup-files \tdon't make backup files");
 	puts("  -c, --config-file CONFIGFILE \tread configuration from CONFIGFILE");
 	puts("  -d, --disable-colors \t\tdon't use colors");
@@ -1506,39 +1531,34 @@ int main(int argc, char *argv[])
 	signal(SIGINT, my_signal_handler);
 	signal(SIGTERM, my_signal_handler);
 
-	while((c = pgetopt(argc, argv, "-a --advance-after-edit -n --no-backup-files "
-					   "-c: --config-file: -d --disable-colors -h --help -v --version")) != -1)
+	while((c = pgetopt(argc, argv, " -n --no-backup-files -c: --config-file: "
+					"-d --disable-colors -h --help -v --version")) != -1)
 	{
 		switch(c)
 		{
 		case 0:
 		case 1:
-			advance_after_edit = 1;
+			make_backup_files = 0;
 			break;
 
 		case 2:
 		case 3:
-			make_backup_files = 0;
+			config_file = poptarg;
 			break;
 
 		case 4:
 		case 5:
-			config_file = poptarg;
+			use_colors = 0;
 			break;
 
 		case 6:
 		case 7:
-			use_colors = 0;
-			break;
-
-		case 8:
-		case 9:
 			print_usage();
 			return 0;
 			break;
 
-		case 10:
-		case 11:
+		case 8:
+		case 9:
 			printf("heme %s\n", heme_version);
 			return 0;
 			break;
